/****************************************************************************
**
** Copyright (C) 2016 Canonical, Ltd. and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the QtSystems module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QINPUTINFO_H
#define QINPUTINFO_H

#include "qsysteminfoglobal.h"
#include <QObject>
#include <QVector>
#include <QMap>
#include <QSocketNotifier>
#include <QDebug>

QT_BEGIN_NAMESPACE

class QInputInfoManagerPrivate;
class QInputDevicePrivate;
class QInputDevice;

class QInputInfoManager;


class Q_SYSTEMINFO_EXPORT QInputDevice : public QObject
{
    Q_OBJECT

    Q_FLAGS(InputType InputTypeFlags)
    Q_PROPERTY(QVariantMap properties READ properties CONSTANT)
    Q_PROPERTY(QInputDevice::InputTypeFlags types READ types CONSTANT)

public:

    enum InputType {
        UnknownType = 0,
        Button = 1,
        Mouse = 2,
        TouchPad = 4,
        TouchScreen = 8,
        Keyboard = 16,
        Switch = 32
    };
    Q_ENUMS(InputType)
    Q_DECLARE_FLAGS(InputTypeFlags, InputType)

    explicit QInputDevice(QObject *parent = 0);
    QString name() const;
    QString identifier() const;
    QList <int> buttons() const; //keys event code
    QList <int> switches() const;
    QList <int> relativeAxes() const;
    QList <int> absoluteAxes() const;
    QInputDevice::InputTypeFlags types() const;
    /*
     * buttons
     * switches
     * rAxes
     * xAxes
     * screenNumber (for touchscreen) ?
     * */
    QVariantMap properties();

private:

    QInputDevicePrivate *d_ptr;
    friend class QInputInfoManagerPrivate;

    void setName(const QString &);
    void setIdentifier(const QString &);
    void addButton(int);
    void addSwitch(int);
    void addRelativeAxis(int);
    void addAbsoluteAxis(int);
    void setTypes(QInputDevice::InputTypeFlags flags);
    QVariantMap deviceProperties;

};


class Q_SYSTEMINFO_EXPORT QInputInfoManager : public QObject
{
    Q_OBJECT

    Q_PROPERTY(int count READ count NOTIFY countChanged)
    Q_PROPERTY(QInputDevice::InputTypeFlags filter READ filter WRITE setFilter NOTIFY filterChanged)
    Q_PROPERTY(QInputDevice * lastAdded READ lastAdded NOTIFY deviceAdded)

public:

    explicit QInputInfoManager(QObject *parent = 0);
    ~QInputInfoManager();

    int count() const;
    int count(const QInputDevice::InputType filter) const;

    void setFilter(QInputDevice::InputTypeFlags filterFlags);
    QInputDevice::InputTypeFlags filter();

    QMap <QString, QInputDevice *> deviceMap();
    QInputDevice *lastAdded();

Q_SIGNALS:

    void ready();
    void deviceAdded(QInputDevice *inputDevice);
    void deviceRemoved(const QString &deviceId);

    void countChanged(int count);
    void filterChanged(QInputDevice::InputTypeFlags filterFlags);

private Q_SLOTS:
    void addedDevice(QInputDevice *devicePath);
    void removedDevice(const QString &deviceId);

private:

    Q_DISABLE_COPY(QInputInfoManager)
#if !defined(QT_SIMULATOR)
    QInputInfoManagerPrivate *const d_ptr;
    Q_DECLARE_PRIVATE(QInputInfoManager)
#endif
    QInputDevice::InputTypeFlags currentFilter;
    QMap <QString, QInputDevice *> currentFilteredMap;
    int filteredCount;

private Q_SLOTS:
    void privateReady();
};

Q_DECLARE_METATYPE(QInputDevice *)
Q_DECLARE_METATYPE(QInputDevice::InputType)
Q_DECLARE_METATYPE(QInputDevice::InputTypeFlags)
Q_DECLARE_OPERATORS_FOR_FLAGS(QInputDevice::InputTypeFlags)

QT_END_NAMESPACE

#endif // QINPUTINFO_H
